unit AssocArray;

{*******************************************************}
{* Tablica asocjacyjna by bidok (michu-bidok@o2.pl)    *}
{* wersja 1.1                                          *}
{* free for non-commercial usage only                  *}
{*******************************************************}

// Jesli chcesz zamiast pointerow uzywac variantow to odkomentuj
// linijke nizej
// {$DEFINE USE_VARIANTS}

interface

{$IFDEF USE_VARIANTS}
uses
  Variants;
{$ELSE}
const
  Null = nil;
{$ENDIF}

type
  PAssocItem = ^TAssocItem;
  TAssocItem = record
    Next: PAssocItem;
    Name: Variant;
    {$IFDEF USE_VARIANTS}
      Value: Variant;
    {$ELSE}
      Value: Pointer;
    {$ENDIF}
  end;

  TAssocArray = class
  protected
    {$IFDEF USE_VARIANTS}
      function GetItem(const Name: Variant): Variant;
      procedure SetItem(const Name: Variant; Value: Variant);
    {$ELSE}
      function GetItem(const Name: Variant): Pointer;
      procedure SetItem(const Name: Variant; Value: Pointer);
    {$ENDIF}
  private
    AList: PAssocItem;
    ALast: PAssocItem;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Clear;
    {$IFDEF USE_VARIANTS}
      procedure AddItem(Name: string; Value: Variant);
    {$ELSE}
      procedure AddItem(Name: string; Value: Pointer);
    {$ENDIF}
    procedure RemItem(Name: string);

    function GetNextItem(var Item: PAssocItem): Boolean;

    {$IFDEF USE_VARIANTS}
      property Items[const Name: Variant]: Variant read GetItem write SetItem; default;
    {$ELSE}
      property Items[const Name: Variant]: Pointer read GetItem write SetItem; default;
    {$ENDIF}
  end;

implementation

constructor TAssocArray.Create;
begin
  inherited Create;

  AList := nil;
  ALast := nil;
end;

destructor TAssocArray.Destroy;
begin
  Clear;

  inherited Destroy;
end;

function TAssocArray.GetNextItem(var Item: PAssocItem): Boolean;
begin
  Result := False;
  if AList = nil then Exit;

  if Item = nil then
  begin
    Item := AList;
    Result := True;
    Exit;
  end;

  if (Item.Next <> nil) then
  begin
    Item := Item^.Next;
    Result := True;
  end;
end;

procedure TAssocArray.Clear;
var
  Temp, Temp2: PAssocItem;
begin
  if AList = nil then Exit;

  Temp := AList;
  while Temp <> nil do
  begin
    Temp2 := Temp^.Next;
    Dispose(Temp);
    Temp := Temp2;
  end;
  AList := nil;
  ALast := nil;
end;

function TAssocArray.GetItem;
var
  Temp: PAssocItem;
begin
  Result := Null;
  if AList = nil then Exit;

  Temp := AList;
  while (Temp <> nil) and (Temp^.Name <> Name) do
    Temp := Temp.Next;

  if Temp = nil then Exit;
  Result := Temp.Value;
end;

procedure TAssocArray.SetItem;
var
  Temp: PAssocItem;
begin
  Temp := AList;
  while (Temp <> nil) and (Temp.Name <> Name) do
    Temp := Temp.Next;

  if Temp = nil then
     AddItem(Name, Value) else
  begin
    if Value = Null then
       RemItem(Name) else
       Temp^.Value := Value;
  end;
end;

procedure TAssocArray.AddItem;
var
  Temp: PAssocItem;
begin
  if Value = Null then Exit;

  New(Temp);
  Temp^.Next := nil;
  Temp^.Name := Name;
  Temp^.Value := Value;

  if (AList = nil) then
  begin
    AList := Temp;
    ALast := Temp;
  end else
  begin
    ALast^.Next := Temp;
    ALast := Temp;
  end;
end;

procedure TAssocArray.RemItem;
var
  Temp, Temp2: PAssocItem;
begin
  if AList = nil then Exit;

  Temp2 := nil;
  Temp := AList;
  while (Temp <> nil) and (Temp.Name <> Name) do
  begin
    Temp2 := Temp;
    Temp := Temp.Next;
  end;

  if Temp = nil then Exit;

  if Temp2 <> nil then
     Temp2^.Next := Temp^.Next else
     AList := Temp^.Next;
  Dispose(Temp);
end;

end.
